CREATE TABLE IF NOT EXISTS `Achievements`
(
    `Id`             BIGINT       NOT NULL AUTO_INCREMENT,
    `Name`           VARCHAR(255) NOT NULL,
    `Description`    VARCHAR(255) NOT NULL,
    `Attribute`      VARCHAR(255) NOT NULL,
    `Operator`       VARCHAR(255) NOT NULL,
    `Value`          VARCHAR(255) NOT NULL,
    `ServerId`       BIGINT,
    `CreatedAt`      DATETIME(6)  NULL DEFAULT CURRENT_TIMESTAMP(6),
    `LastModifiedAt` DATETIME(6)  NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
    PRIMARY KEY (`Id`),
    FOREIGN KEY (`ServerId`) REFERENCES `Servers` (`ServerId`)
);



CREATE TABLE IF NOT EXISTS `AchievementsHistory`
(
    `Id`          BIGINT(20)   NOT NULL,
    `Name`        VARCHAR(255) NOT NULL,
    `Description` VARCHAR(255) NOT NULL,
    `Attribute`   VARCHAR(255) NOT NULL,
    `Operator`    VARCHAR(255) NOT NULL,
    `Value`       VARCHAR(255) NOT NULL,
    `ServerId`    BIGINT,
    `Deleted`     BOOL DEFAULT FALSE,
    `DateFrom`    DATETIME(6)  NOT NULL,
    `DateTo`      DATETIME(6)  NOT NULL
);



CREATE TABLE IF NOT EXISTS `UserGotAchievements`
(
    `Id`             BIGINT      NOT NULL AUTO_INCREMENT,
    `UserId`         BIGINT,
    `AchievementId`  BIGINT,
    `CreatedAt`      DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
    `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
    PRIMARY KEY (`Id`),
    FOREIGN KEY (`UserId`) REFERENCES `Users` (`UserId`),
    FOREIGN KEY (`AchievementId`) REFERENCES `Achievements` (`Id`)
);


ALTER TABLE Users
    ADD MessageCount BIGINT NOT NULL DEFAULT 0 AFTER XP;

ALTER TABLE Users
    ADD ReactionCount BIGINT NOT NULL DEFAULT 0 AFTER XP;

ALTER TABLE UsersHistory
    ADD MessageCount BIGINT NOT NULL DEFAULT 0 AFTER XP;

ALTER TABLE UsersHistory
    ADD ReactionCount BIGINT NOT NULL DEFAULT 0 AFTER XP;

DROP TRIGGER IF EXISTS `TR_AchievementsUpdate`;


CREATE TRIGGER `TR_AchievementsUpdate`
    AFTER UPDATE
    ON `Achievements`
    FOR EACH ROW
BEGIN
    INSERT INTO `AchievementsHistory` (`Id`, `Name`, `Description`, `Attribute`, `Operator`, `Value`, `ServerId`,
                                       `DateFrom`, `DateTo`)
    VALUES (OLD.Id, OLD.Name, OLD.Description, OLD.Attribute, OLD.Operator, OLD.Value, OLD.ServerId, OLD.LastModifiedAt,
            CURRENT_TIMESTAMP(6));
END;


DROP TRIGGER IF EXISTS `TR_AchievementsDelete`;


CREATE TRIGGER `TR_AchievementsDelete`
    AFTER DELETE
    ON `Achievements`
    FOR EACH ROW
BEGIN
    INSERT INTO `AchievementsHistory` (`Id`, `Name`, `Description`, `Attribute`, `Operator`, `Value`, `ServerId`,
                                       `Deleted`, `DateFrom`, `DateTo`)
    VALUES (OLD.Id, OLD.Name, OLD.Description, OLD.Attribute, OLD.Operator, OLD.Value, OLD.ServerId, TRUE,
            OLD.LastModifiedAt, CURRENT_TIMESTAMP(6));
END;


