from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.abc.user_warnings_repository_abc import UserWarningsRepositoryABC
from bot_data.model.user_warnings import UserWarnings


class UserWarningsRepositoryService(UserWarningsRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        users: UserRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._users = users
        UserWarningsRepositoryABC.__init__(self)

    @staticmethod
    def _get_value_from_result(value: any) -> Optional[any]:
        if isinstance(value, str) and "NULL" in value:
            return None

        return value

    def _from_result(self, sql_result: tuple) -> UserWarnings:
        user = self._users.get_user_by_id(self._get_value_from_result(sql_result[2]))
        author = None
        author_id = self._get_value_from_result(sql_result[3])
        if author_id is not None:
            author = self._users.get_user_by_id(author_id)

        return UserWarnings(
            self._get_value_from_result(sql_result[1]),
            user,
            author,
            self._get_value_from_result(sql_result[4]),
            self._get_value_from_result(sql_result[5]),
            id=self._get_value_from_result(sql_result[0]),
        )

    def get_user_warnings(self) -> List[UserWarnings]:
        self._logger.trace(__name__, f"Send SQL command: {UserWarnings.get_select_all_string()}")
        return List(
            UserWarnings,
            [self._from_result(warning) for warning in self._context.select(UserWarnings.get_select_all_string())],
        )

    def get_user_warnings_by_id(self, id: int) -> UserWarnings:
        self._logger.trace(__name__, f"Send SQL command: {UserWarnings.get_select_by_id_string(id)}")
        return self._from_result(self._context.select(UserWarnings.get_select_by_id_string(id))[0])

    def get_user_warnings_by_server_id(self, server_id: int) -> List[UserWarnings]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserWarnings.get_select_by_server_id_string(server_id)}",
        )

        return List(
            UserWarnings,
            [
                self._from_result(warning)
                for warning in self._context.select(UserWarnings.get_select_by_server_id_string(server_id))
            ],
        )

    def get_user_warnings_by_user_id(self, user_id: int) -> List[UserWarnings]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserWarnings.get_select_by_user_id_string(user_id)}",
        )

        return List(
            UserWarnings,
            [
                self._from_result(warning)
                for warning in self._context.select(UserWarnings.get_select_by_user_id_string(user_id))
            ],
        )

    def add_user_warnings(self, user_warnings: UserWarnings):
        self._logger.trace(__name__, f"Send SQL command: {user_warnings.insert_string}")
        self._context.cursor.execute(user_warnings.insert_string)

    def update_user_warnings(self, user_warnings: UserWarnings):
        self._logger.trace(__name__, f"Send SQL command: {user_warnings.udpate_string}")
        self._context.cursor.execute(user_warnings.udpate_string)

    def delete_user_warnings(self, user_warnings: UserWarnings):
        self._logger.trace(__name__, f"Send SQL command: {user_warnings.delete_string}")
        self._context.cursor.execute(user_warnings.delete_string)
